@@ -21,11 +21,13 @@
  * USA
  */
 
+#include "pyglib-private.h"
+#include "pygobject-private.h"
+#include "pyginterface.h"
 #include "pygi-private.h"
 #include "pygi.h"
 #include "pyglib.h"
 
-#include <pygobject.h>
 #include <pyglib-python-compat.h>
 
 PyObject *PyGIDeprecationWarning;
@@ -627,6 +629,8 @@ static struct PyGI_API CAPI = {
 PYGLIB_MODULE_START(_gi, "_gi")
 {
     PyObject *api;
+    PyObject *_glib_module;
+    PyObject *_gobject_module;
 
     /* Always enable Python threads since we cannot predict which GI repositories
      * might accept Python callbacks run within non-Python threads or might trigger
@@ -635,13 +639,30 @@ PYGLIB_MODULE_START(_gi, "_gi")
      */
     PyEval_InitThreads ();
 
-    if (pygobject_init (-1, -1, -1) == NULL) {
+    _glib_module = pyglib__glib_module_create ();
+    if (_glib_module == NULL) {
         return PYGLIB_MODULE_ERROR_RETURN;
     }
+    /* In Python 2.x, pyglib_..._module_create returns a borrowed reference and
+     * PyModule_AddObject steals a reference. Ensure we don't share a reference
+     * between sys.modules and gi._gi._glib by incrementing the ref count here.
+     * Note that we don't add this to the PYGLIB_MODULE_START macro because that
+     * would cause a leak for the main module gi._gi */
+    if (PY_MAJOR_VERSION < 3) {
+        Py_INCREF (_glib_module);
+    }
+    PyModule_AddObject (module, "_glib", _glib_module);
+    PyModule_AddStringConstant(module, "__package__", "gi._gi");
 
-    if (_pygobject_import() < 0) {
+    _gobject_module = pyglib__gobject_module_create ();
+    if (_gobject_module == NULL) {
         return PYGLIB_MODULE_ERROR_RETURN;
     }
+    if (PY_MAJOR_VERSION < 3) {
+        Py_INCREF (_gobject_module);
+    }
+    PyModule_AddObject (module, "_gobject", _gobject_module);
+    PyModule_AddStringConstant(module, "__package__", "gi._gi");
 
     _pygi_repository_register_types (module);
     _pygi_info_register_types (module);
