@@ -422,9 +422,9 @@ static gboolean _caller_alloc (PyGIInvokeState *state,
                 state->args[arg_count]->v_pointer = g_malloc0 (size);
         }
     } else if (arg_cache->type_tag == GI_TYPE_TAG_ARRAY) {
-        PyGISequenceCache *seq_cache = (PyGISequenceCache *)arg_cache;
+        PyGIArgGArray *array_cache = (PyGIArgGArray *)arg_cache;
 
-        state->out_args[out_count].v_pointer = g_array_new (TRUE, TRUE, seq_cache->item_size);
+        state->out_args[out_count].v_pointer = g_array_new (TRUE, TRUE, array_cache->item_size);
         state->args[arg_count] = &state->out_args[out_count];
     } else {
         return FALSE;
@@ -720,7 +720,7 @@ _wrap_g_callable_info_invoke (PyGIBaseInfo *self, PyObject *py_args,
                               PyObject *kwargs)
 {
     if (self->cache == NULL) {
-        self->cache = _pygi_callable_cache_new (self->info, FALSE);
+        self->cache = pygi_callable_cache_new (self->info, FALSE);
         if (self->cache == NULL)
             return NULL;
     }
